//*****************************************************************************
//*
//*
//*		WinAudio.c
//*
//*
//*****************************************************************************
//
//	A audio driver for windows
//	Written by Anton Zechen (C)2001
//
//	include	winmm.lib
//
#include	<io.h>
#include	<stdio.h>
#include	<windows.h>
#include	<windowsx.h>
#include	<mmsystem.h>
#include	"WinAudio.h"

/*	Example:

	void *data;
	AudioOpen(44100,16,2,0x4000);
	AudioSetMainVolume(32000);
	AudioSetVolume(32000);
	while(playing)
		{
		data=AudioWritePtr();
		memcpy(data,???,0x4000);
		AudioWrite(0x4000);
		}
	AudioWait(2000);
	AudioClose();

*/

#ifndef	_WIN32
#ifndef	_WIN64
#error	"This Audio-Driver is only for Windows !"
#endif
#endif

#pragma	comment( lib, "winmm.lib" )


#define		AUDIO_BUFFER_MAX	16				// Max buffer count
#define		AUDIO_BUFFER_MIN	12				// Min buffer count
#define		AUDIO_DELAY			8				// Min buffer count
#define		AUDIO_WAIT			2000			// Wait for writeop

static  WAVEOUTCAPS		 sAudioCaps;
static  CRITICAL_SECTION hAudioCs    ={0};
static  WAVEFORMATEX	 sAudioWm	 ={0,0,0,0,0,0,0};
static  HWAVEOUT         hAudioId    = 0;  		// Audio IDize
static  HANDLE           hAudioEvent = 0;  		// Event handle
static  HANDLE           hAudioThread= 0;  		// Thread handle
static	LONG			 lAudioDone  = 0;		// Gespielte Bytes
static  WAVEHDR         *aAudioHdr [AUDIO_BUFFER_MAX];
static	char 		    *aAudioPtr [AUDIO_BUFFER_MAX];
static	unsigned int	 aAudioSize[AUDIO_DELAY     ];
static	unsigned int	 uAudioPlay=AUDIO_BUFFER_MAX;
static	unsigned int	 uAudioWrOk  = 0;		// Write position
static	unsigned int	 uAudioWrite = 0;		// Write position
static	unsigned int	 uAudioCount = 0;		// Buffer count
static	unsigned int	 uAudioDelay = 0;		// Buffer count
static  unsigned int     uAudioBufLn = 0x4000;	// Buffer size
static  unsigned int     uAudioStop  = 0;		// Stops AutioWrite???
static  int volatile	 iAudioThEnd = 0;  		// Thread ends
static	int (_cdecl     *pAudioCallback)(void*,void*,int);


//*****************************************************************************
//*
//*		AudioOutProc
//*
//*****************************************************************************
static void CALLBACK AudioOutProc(HWAVEOUT hOut,UINT uMsg,DWORD dwInstance,DWORD dwParam1,DWORD dwParam2)
{
WAVEHDR	    *pHeader;
int			iDelta;


	switch(uMsg)
		{
	case WOM_DONE:

			pHeader=(WAVEHDR*)dwParam1;

			EnterCriticalSection(&hAudioCs);

			InterlockedExchangeAdd(&lAudioDone,pHeader->dwBufferLength);

			   iDelta= uAudioPlay-pHeader->dwUser;
			if(iDelta<0)iDelta += uAudioCount;
			if(iDelta>((int)uAudioCount>>1))
				{
				   uAudioPlay=pHeader->dwUser;
				if(uAudioPlay>=uAudioCount)uAudioPlay=0;

				   iDelta=uAudioPlay -uAudioWrite;
				if(iDelta< 0)iDelta += uAudioCount;
				if(iDelta>=2)SetEvent(hAudioEvent);
				}

			LeaveCriticalSection(&hAudioCs);

			break;

//	case WOM_CLOSE:	break;
//	case WOM_OPEN:	break;
		}

}


//*****************************************************************************
//*
//*		AudioOpen
//*
//*****************************************************************************
//	Opens a new audio putput
//	uFrequ		: Is the sampling frequency (8000,11025,22050,44100)
//	uBits		: Is the bit count for one sample (8,12,16)
//	uChannels	: Is the number of output channels (1=mono,2=stereo)
//	uBufferSize	: Is the maximum buffer size for one write operation
//	Return	0	: All is ok.
//			1	: Mode error
//			2	: Memmory error
//			3	: Global error
int AudioOpen(unsigned uFrequ,unsigned uBits,unsigned uChannels,unsigned uBufferSize)
{
int				iBytePerSample;
MMRESULT		uRes;


	if(hAudioId)AudioClose();

	if(uBits  &0x03)return 1;
	if(uBits    >32)return 1;
	if(uChannels>32)return 1;
	if(uFrequ>48000)return 1;

	iBytePerSample=((uBits*uChannels)+7)>>3;			// set mode

	sAudioWm.wFormatTag		 = WAVE_FORMAT_PCM;
	sAudioWm.nChannels		 = uChannels;
	sAudioWm.nSamplesPerSec	 = uFrequ;
	sAudioWm.nAvgBytesPerSec = uFrequ*iBytePerSample;
	sAudioWm.nBlockAlign	 = iBytePerSample;
	sAudioWm.wBitsPerSample	 = uBits;
	sAudioWm.cbSize			 = 0;

	uRes=waveOutOpen(&hAudioId, WAVE_MAPPER ,			// open device
					 &sAudioWm,(DWORD)AudioOutProc,
					 0x00000000,CALLBACK_FUNCTION);

	if(uRes || !hAudioId)
		{
		memset(&sAudioWm,0,sizeof(sAudioWm));
		return 1;
		}

	if(uBufferSize)uAudioBufLn=uBufferSize;
														// alloc buffers
	uAudioDelay = 0;
	uAudioWrite = 0;
	uAudioStop  = 0;
	lAudioDone	= 0;


	for(uAudioCount=0;uAudioCount<AUDIO_BUFFER_MAX;uAudioCount++)
		{
		aAudioHdr[uAudioCount] = (LPWAVEHDR)GlobalAllocPtr(GMEM_MOVEABLE|GMEM_SHARE,sizeof(WAVEHDR)+uAudioBufLn);

		if(!aAudioHdr[uAudioCount])
			{
			break;
			}

		aAudioHdr[uAudioCount]->dwFlags		    = WHDR_DONE;
		aAudioHdr[uAudioCount]->lpData		    = (LPSTR) aAudioHdr[uAudioCount]+ sizeof(WAVEHDR);
		aAudioHdr[uAudioCount]->dwBufferLength  = uAudioBufLn;
		aAudioHdr[uAudioCount]->dwUser			= uAudioCount;
		aAudioHdr[uAudioCount]->dwBytesRecorded = 0;
		aAudioHdr[uAudioCount]->dwLoops			= 1;
		aAudioHdr[uAudioCount]->lpNext			= 0;
		aAudioPtr[uAudioCount]=(char*)(aAudioHdr[uAudioCount])+sizeof(WAVEHDR);

		if(!waveOutPrepareHeader(hAudioId,aAudioHdr[uAudioCount],sizeof(WAVEHDR)))
			{
			continue;
			}

		GlobalFreePtr((LPSTR) aAudioHdr[uAudioCount]);
		break;
		}

	uAudioPlay  = uAudioCount-1;

	if(uAudioCount<AUDIO_BUFFER_MIN)					// memmory error ?
		{
		AudioClose();
		return 2;
		}

	InitializeCriticalSection(&hAudioCs);

		hAudioEvent=CreateEvent(0,TRUE,TRUE,0);
	if(!hAudioEvent)
		{
		AudioClose();
		return 3;
		}

	if(waveOutGetDevCaps((UINT)hAudioId,&sAudioCaps,sizeof(WAVEOUTCAPS)))
		{
		memset(&sAudioCaps,0,sizeof(sAudioCaps));
		}

	InitializeCriticalSection(&hAudioCs);
	waveOutReset(hAudioId);
	uAudioWrOk = 1;


return 0;
}

//*****************************************************************************
//*
//*		AudioClose
//*
//*****************************************************************************
//	Close an audio putput
int AudioClose()
{
int	 iErr=0;


	uAudioWrOk=0;

	if(!hAudioId)return 1;

	AudioStop();										// reset audio output
	memset(&sAudioWm,0,sizeof(sAudioWm));
	DeleteCriticalSection(&hAudioCs);

	while(uAudioCount>0)								// release buffers
		{
		uAudioCount--;
		if(waveOutUnprepareHeader(hAudioId ,aAudioHdr[uAudioCount],sizeof(WAVEHDR)))iErr=1;
		if(GlobalFreePtr( (LPSTR) aAudioHdr[uAudioCount]))iErr=1;
		aAudioHdr[uAudioCount]=0;
		aAudioPtr[uAudioCount]=0;
		}

	if( waveOutClose(hAudioId   ))iErr=1;				// close device
	if(!CloseHandle (hAudioEvent))iErr=1;

	hAudioId	= 0;
	uAudioWrite	= 0;
	uAudioDelay	= 0;
	uAudioPlay	= uAudioCount-1;


return iErr;
}

//*****************************************************************************
//*
//*		AudioInfo
//*
//*****************************************************************************
//	Get infos of the opened audio output
//	pFreq		: Target for the sampling frequency (8000,11025,22050,44100)
//	pBits		: Target for the bit-count for one value (8,12,16)
//	pChannels	: Target for the number of output-channels (1=mono,2=stereo)
//	pBufferSize	: Target for the maximum buffer size for one write operation
//  If a pointer is zero then the there will not saved a value.
//	Return	0	: Audio is not open
//			1	: Audio is open
int	AudioInfo(unsigned *pFreq,unsigned *pBits,unsigned *pChannels,unsigned  *pBufferSize )
{


	if(pFreq      )*pFreq		= sAudioWm.nSamplesPerSec;
	if(pBits      )*pBits		= sAudioWm.wBitsPerSample;
	if(pChannels  )*pChannels   = sAudioWm.nChannels;
	if(pBufferSize)*pBufferSize = uAudioBufLn;


return (hAudioId)? 1:0;
}

//*****************************************************************************
//*
//*		AudioWritePtr
//*
//*****************************************************************************
//	Returns a pointer to the next write buffer
void *AudioWritePtr()
{

	if(!hAudioId)return 0;


return aAudioPtr[uAudioWrite];
}

//*****************************************************************************
//*
//*		AudioSetVolume
//*
//*****************************************************************************
//	Set the output volume   (uVolume=0 to 0xFFFF)
int AudioSetVolume(unsigned uVolume)
{
int		uNew;


	if(!hAudioId)return -1;

	if(!(sAudioCaps.dwSupport&WAVECAPS_VOLUME))
		{
		return -1;
		}

	if(uVolume>0xFFFF)
		{
		uVolume = 0xFFFF;
		}

	uNew = uVolume;

	if(sAudioCaps.dwSupport&WAVECAPS_LRVOLUME)
		{
		uVolume |= uVolume<<16;
		}

	if(waveOutSetVolume(hAudioId,uVolume))
		{
		return -1;
		}


return uNew;
}

//*****************************************************************************
//*
//*		AudioGetVolume
//*
//*****************************************************************************
//	Get the output volume
int AudioGetVolume()
{
DWORD	uVolume;


	if(!hAudioId)return -1;

	if(~sAudioCaps.dwSupport&WAVECAPS_VOLUME  )return -1;
	if( waveOutGetVolume(hAudioId,&uVolume   ))return -1;
	if( sAudioCaps.dwSupport&WAVECAPS_LRVOLUME)
		{
		uVolume=((uVolume&0xFFFF)+(uVolume>>16))>>1;
		}


return uVolume&0xFFFF;
}

//*****************************************************************************
//*
//*		AudioStop
//*
//*****************************************************************************
//	Stops the playing and clears the buffers
int AudioStop()
{
unsigned  uPos;


 	if(!hAudioId)return 1;

	uAudioStop = 1;
	uAudioWrOk = 0;
	SetEvent(hAudioEvent);

	waveOutReset(hAudioId);

	if(uAudioWrite!=0 && uAudioPlay!=uAudioCount-1)
		{
		for(uPos=0;uPos<uAudioCount;uPos++)					// clears all buffers
			{
			memset(aAudioHdr[uPos]->lpData,0,uAudioBufLn);
			break;
			}
		}

	uAudioPlay  = uAudioCount-1;
	uAudioWrite = 0;
	uAudioDelay = 0;

	SetEvent(hAudioEvent);
	uAudioWrOk  = 1;

	if(hAudioThread)										// close thread
		{
		for(uPos=0;!iAudioThEnd && uPos<10;uPos++)
			{
			Sleep(100);
			}

		if(!iAudioThEnd)
			{
			TerminateThread(hAudioThread,0);
			Sleep(100);
			}

		CloseHandle(hAudioThread);

		hAudioThread = 0;
		}


return 0;
}

//*****************************************************************************
//*
//*		AudioReset
//*
//*****************************************************************************
//	Clears the buffers
int AudioReset()
{
unsigned  uPos;


 	if(!hAudioId)return 1;

	uAudioStop = 1;
	uAudioWrOk = 0;
	SetEvent(hAudioEvent);

	waveOutReset(hAudioId);

	if(uAudioWrite!=0 && uAudioPlay!=uAudioCount-1)
		{
		for(uPos=0;uPos<uAudioCount;uPos++)					// clears all buffers
			{
			memset(aAudioHdr[uPos]->lpData,0,uAudioBufLn);
			break;
			}
		}

	uAudioPlay  = uAudioCount-1;
	uAudioWrite = 0;
	uAudioDelay = 0;

	SetEvent(hAudioEvent);
	uAudioWrOk  = 1;



return 0;
}

//*****************************************************************************
//*
//*		AudioPause
//*
//*****************************************************************************
//	Stops  playing if iPause != 0
//	Resume playing if iPause == 0
int AudioPause(int iPause)
{
MMRESULT	uRet;


	if(!hAudioId)return 1;

	if(iPause)uRet=waveOutPause  (hAudioId);
	else	  uRet=waveOutRestart(hAudioId);


return (uRet)? 1:0;
}

//*****************************************************************************
//*
//*		AudioIsReady
//*
//*****************************************************************************
//	Returns 0 if there is any sample playing
int AudioIsReady()
{
unsigned	uWrite;
unsigned 	uPlay;


	if(!hAudioId)return 1;

	uPlay  = uAudioPlay;
	uWrite = uAudioWrite;

	if(uWrite<uPlay)uWrite+=uAudioCount;


return (uWrite==uPlay+1)? 1:0;
}

//*****************************************************************************
//*
//*		AudioWait
//*
//*****************************************************************************
//	Wait to the end of Playing
//	Milisec is the maximum wait time , -1 is for ever
int AudioWait(int iWait)
{
DWORD		uTick;


	if(!hAudioId)return 1;


	if(iWait<0)iWait=INFINITE;
	uTick=GetTickCount()+iWait;

	if(WaitForSingleObject(hAudioEvent,iWait)==WAIT_FAILED)return 1;

	for(;;) 											// wait to ready
		{
		if(AudioIsReady())return 0;
		if(iWait!=INFINITE)
		if(uTick<GetTickCount())break;
		Sleep(100);
		}


return 1;
}

//*****************************************************************************
//*
//*		AudioDone
//*
//*****************************************************************************
//	Ergibt die Anzahl der abgespielten Bytes vom Stream
int	AudioDone()
{
LONG	lDone;


	lDone = InterlockedExchange(&lAudioDone,0);


return lDone;
}

//*****************************************************************************
//*
//*		AudioDelay
//*
//*****************************************************************************
//	Return	the playing delay time in seconds /16
int AudioDelay()
{
unsigned	uWrite;
unsigned	uPlay;
unsigned	uLen;


	if(!hAudioId)return 0;


	uWrite = uAudioWrite;
	uPlay  = uAudioPlay+1;
	uLen   = 0;


	for(;;)
		{
		if(uPlay>=uAudioCount)uPlay=0;
		if(uPlay==uWrite)break;

		uLen+=aAudioHdr[uPlay]->dwBufferLength;
		uPlay++;
		}

	uLen /= (sAudioWm.nSamplesPerSec>>4)*sAudioWm.nBlockAlign;


return uLen;
}

//*****************************************************************************
//*
//*		AudioWrite
//*
//*****************************************************************************
//	Return	0	: All is ok.
//			1	: Timeout
//			2	: If Count is bigger than buffer-size
//			3	: Write error
//			4	: Mode Error
int AudioWrite(unsigned uCount)
{
int			 iDelta,i,iPos;
WAVEHDR		*pHeader;



	if(!uAudioWrOk)return 4;
	if(uCount>uAudioBufLn)return 2;


	if(WaitForSingleObject(hAudioEvent,AUDIO_WAIT)==WAIT_FAILED)
		{
		return 1;
		}

	if(uAudioDelay<AUDIO_DELAY)
		{
		aAudioSize[uAudioDelay]=uCount;
		uAudioDelay++;

		EnterCriticalSection(&hAudioCs);				// increase write pos
		   uAudioWrite++;
		if(uAudioWrite>=uAudioCount)uAudioWrite=0;
		LeaveCriticalSection(&hAudioCs);
		return 0;
		}

	if(uAudioDelay==AUDIO_DELAY)
		{
		uAudioDelay++;

		for(i=AUDIO_DELAY;i>0;i--)
			{
			   iPos=uAudioWrite-i;
			if(iPos<0)iPos+=uAudioCount;
			pHeader=aAudioHdr[iPos];
			pHeader->dwBufferLength=aAudioSize[AUDIO_DELAY-i];
			if(waveOutWrite(hAudioId,pHeader,sizeof(WAVEHDR)))return 3;
			}
		}

	pHeader = aAudioHdr[uAudioWrite];
	pHeader->dwBufferLength	= uCount;

	if(waveOutWrite(hAudioId,pHeader,sizeof(WAVEHDR)))
		{
		return 3;
		}


	EnterCriticalSection(&hAudioCs);					// increase write pos

	   uAudioWrite++;
	if(uAudioWrite>=uAudioCount)uAudioWrite=0;

	   iDelta=uAudioPlay-uAudioWrite;
	if(iDelta<0)iDelta += uAudioCount;
	if(iDelta<2)ResetEvent(hAudioEvent);

	LeaveCriticalSection(&hAudioCs);



return 0;
}

//*****************************************************************************
//*
//*		AudioSetMainVolume
//*
//*****************************************************************************
//	Set the output volume   (uVolume=0 to 0xFFFF)
//	Returns -1 if an error was detected
int AudioSetMainVolume(unsigned uVolume)
{
MIXERCONTROLDETAILS_UNSIGNED sMxcdVolume;
MIXERCONTROLDETAILS			 sMxcd;
MIXERLINECONTROLS			 sMxlc;
MIXERCONTROL				 sMxc;
MIXERLINE					 sMxl;
HMIXER						 hMixer;
MIXERCAPS					 hMxCaps;
MMRESULT					 uResult;
UINT						 uNumMixers;



	   uNumMixers = mixerGetNumDevs();
	if(uNumMixers<=0)return -1;
	   hMixer = NULL;
	   ZeroMemory(&hMxCaps,sizeof(MIXERCAPS));


	   uResult  = mixerOpen(&hMixer,0,0,0,MIXER_OBJECTF_MIXER);
	if(uResult != MMSYSERR_NOERROR)return -1;
	   uResult  = mixerGetDevCaps((UINT)hMixer,&hMxCaps, sizeof(MIXERCAPS));
	if(uResult != MMSYSERR_NOERROR){mixerClose(hMixer);return -1;}


	sMxl.cbStruct        = sizeof(MIXERLINE);
	sMxl.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_SPEAKERS;

	   uResult  = mixerGetLineInfo((HMIXEROBJ)hMixer,&sMxl,MIXER_OBJECTF_HMIXER|MIXER_GETLINEINFOF_COMPONENTTYPE);
	if(uResult != MMSYSERR_NOERROR){mixerClose(hMixer);return -1;}


	sMxlc.cbStruct       = sizeof(MIXERLINECONTROLS);			// get dwControlID
	sMxlc.dwLineID       = sMxl.dwLineID;
	sMxlc.dwControlType  = MIXERCONTROL_CONTROLTYPE_VOLUME;
	sMxlc.cControls      = 1;
	sMxlc.cbmxctrl       = sizeof(MIXERCONTROL);
	sMxlc.pamxctrl       = &sMxc;

	   uResult  = mixerGetLineControls((HMIXEROBJ)hMixer,&sMxlc,MIXER_OBJECTF_HMIXER|MIXER_GETLINECONTROLSF_ONEBYTYPE);
	if(uResult != MMSYSERR_NOERROR){mixerClose(hMixer);return -1;}

	if(uVolume<sMxc.Bounds.dwMinimum)uVolume=sMxc.Bounds.dwMinimum;
	if(uVolume>sMxc.Bounds.dwMaximum)uVolume=sMxc.Bounds.dwMaximum;


	sMxcdVolume.dwValue	 = uVolume;
	sMxcd.cbStruct		 = sizeof(MIXERCONTROLDETAILS);
	sMxcd.dwControlID	 = sMxc.dwControlID;
	sMxcd.cChannels		 = 1;
	sMxcd.cMultipleItems = 0;
	sMxcd.cbDetails		 = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
	sMxcd.paDetails		 = &sMxcdVolume;

	   uResult  = mixerSetControlDetails((HMIXEROBJ)hMixer,&sMxcd,MIXER_OBJECTF_HMIXER|MIXER_SETCONTROLDETAILSF_VALUE);
	if(uResult != MMSYSERR_NOERROR){mixerClose(hMixer);return -1;}

	mixerClose(hMixer);



return uVolume;
}

//*****************************************************************************
//*
//*		AudioGetMainVolume
//*
//*****************************************************************************
//	Get the output volume   (uVolume=0 to 0xFFFF)
int AudioGetMainVolume()
{
MIXERCONTROLDETAILS_UNSIGNED sMxcdVolume;
MIXERCONTROLDETAILS			 sMxcd;
MIXERLINECONTROLS			 sMxlc;
MIXERCONTROL				 sMxc;
MIXERLINE					 sMxl;
HMIXER						 hMixer;
MIXERCAPS					 hMxCaps;
MMRESULT					 uResult;
UINT						 uNumMixers;



	   uNumMixers = mixerGetNumDevs();
	if(uNumMixers<=0)return -1;
	   hMixer = NULL;
	   ZeroMemory(&hMxCaps,sizeof(MIXERCAPS));


	   uResult  = mixerOpen(&hMixer,0,0,0,MIXER_OBJECTF_MIXER);
	if(uResult != MMSYSERR_NOERROR)return -1;
	   uResult  = mixerGetDevCaps((UINT)hMixer,&hMxCaps, sizeof(MIXERCAPS));
	if(uResult != MMSYSERR_NOERROR){mixerClose(hMixer);return -1;}


	sMxl.cbStruct        = sizeof(MIXERLINE);
	sMxl.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_SPEAKERS;

	   uResult  = mixerGetLineInfo((HMIXEROBJ)hMixer,&sMxl,MIXER_OBJECTF_HMIXER|MIXER_GETLINEINFOF_COMPONENTTYPE);
	if(uResult != MMSYSERR_NOERROR){mixerClose(hMixer);return -1;}


	sMxlc.cbStruct       = sizeof(MIXERLINECONTROLS);			// get dwControlID
	sMxlc.dwLineID       = sMxl.dwLineID;
	sMxlc.dwControlType  = MIXERCONTROL_CONTROLTYPE_VOLUME;
	sMxlc.cControls      = 1;
	sMxlc.cbmxctrl       = sizeof(MIXERCONTROL);
	sMxlc.pamxctrl       = &sMxc;

	   uResult  = mixerGetLineControls((HMIXEROBJ)hMixer,&sMxlc,MIXER_OBJECTF_HMIXER|MIXER_GETLINECONTROLSF_ONEBYTYPE);
	if(uResult != MMSYSERR_NOERROR){mixerClose(hMixer);return -1;}



	sMxcd.cbStruct		 = sizeof(MIXERCONTROLDETAILS);
	sMxcd.dwControlID	 = sMxc.dwControlID;
	sMxcd.cChannels		 = 1;
	sMxcd.cMultipleItems = 0;
	sMxcd.cbDetails		 = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
	sMxcd.paDetails		 = &sMxcdVolume;


	   uResult  = mixerGetControlDetails((HMIXEROBJ)hMixer,&sMxcd, MIXER_OBJECTF_HMIXER|MIXER_GETCONTROLDETAILSF_VALUE);
	if(uResult != MMSYSERR_NOERROR){mixerClose(hMixer);return -1;}


	mixerClose(hMixer);


return sMxcdVolume.dwValue;
}


//*****************************************************************************
//*
//*		AudioSoftStop
//*
//*****************************************************************************
//	Stop Playing but this funktion waits for the end of the current buffer
int AudioSoftStop()
{
int		 i,iExit;
void	*pPtr;


	if(!hAudioId)return 1;

	i =sAudioWm.nSamplesPerSec/4;
	if(sAudioWm.nChannels>1)i<<=1;
	if(sAudioWm.wBitsPerSample>8)i<<=1;
	if(i>(int)uAudioBufLn)i=uAudioBufLn;

	pPtr=AudioWritePtr();
	memset(pPtr,0,i);
	AudioWrite(i);

	AudioWait(3000);
	Sleep(100);
	iExit=AudioStop();


return iExit;
}


//*****************************************************************************
//*
//*		AudioIsOpen
//*
//*****************************************************************************
//	Returns 1 ist the Audio Driver ist open
int AudioIsOpen()
{
return (hAudioId)? 1:0;
}


//*****************************************************************************
//*
//*		AudioProcThread
//*
//*****************************************************************************
static DWORD WINAPI AudioProcThread(LPVOID pParam)
{
int		iExit;


	iAudioThEnd = 0;
	iExit       = AudioWriteProc(pAudioCallback,pParam);
	iAudioThEnd = 1;


return iExit;
}

//*****************************************************************************
//*
//*		AudioFileRead
//*
//*****************************************************************************
static int _cdecl AudioFileRead(void *pFile,void *pBuffer,int iSize)
{
return fread(pBuffer,1,iSize,(FILE*)pFile);
}

//*****************************************************************************
//*
//*		AudioIoRead
//*
//*****************************************************************************
static int _cdecl AudioIoRead(void *pFile,void *pBuffer,int iSize)
{
return read((int)pFile,pBuffer,iSize);
}

//*****************************************************************************
//*
//*		AudioWriteIo
//*
//*****************************************************************************
//	Reads the playing data from the file which is open with Handle
//	AudioWrite uses the read(Handle,void*,int)  function to read the Data
//	Returns the readed bytes or -1 if an error occurs
int AudioWriteIo(int iHandle)
{
return AudioWriteProc(AudioIoRead,(void*)iHandle);
}

//*****************************************************************************
//*
//*		AudioWriteFile
//*
//*****************************************************************************
//	Writes the playing data from the file which is open with pFile
//	AudioWrite uses the AudioFileRead(File,void*,int)  funktion to read the data
//	Returns the readed bytes or -1 if an error occurs
int AudioWriteFile(FILE *pFile)
{
return AudioWriteProc(AudioFileRead,pFile);
}

//*****************************************************************************
//*
//*		AudioWriteProc
//*
//*****************************************************************************
//	Writes the playing data from the funktion pProc
//	AudioWrite uses the pProc(pParam,void* Buffer,int Count) to read the data
//	Returns the readed bytes or -1 if an error occurs
int AudioWriteProc(AudioRdProc pProc,void *pParam)
{
void   *pData;
int		iCount;
int		iAll=0;


		uAudioStop=0;
	if(!hAudioId)return -1;


	while(!uAudioStop && hAudioId)					// while no break
		{
		pData  = AudioWritePtr();
		iCount = pProc(pParam,pData,uAudioBufLn);
		if(iCount<=0)break;
		AudioWrite(iCount);
		iAll+=iCount;
		}

return iAll;
}



//*****************************************************************************
//*
//*		AudioWriteIoThread
//*
//*****************************************************************************
//	This funktion starts an thread which read the Data from a file
//	which is opened with Handle
//	AudioWriteThread uses the write(Handle,void*,int) funktion to read the Data
//	The Data will be readed in Background
int AudioWriteIoThread(int iHandle)
{

	if(!hAudioId)return -1;

	if(hAudioThread)AudioStop();					// close old thread

	pAudioCallback  = AudioIoRead;
	hAudioThread	= CreateThread(0,1024,AudioProcThread,(void*)iHandle,0,0);


return 0;
}

//*****************************************************************************
//*
//*		AudioWriteFileThread
//*
//*****************************************************************************
//	This funktion starts an thread which read the Data from a file
//	which is opened with pFile
//	AudioWriteProcThread uses the pProc(pParam,void *pBuffer,int iCount) to read the Data
//	The Data will be readed in Background
int AudioWriteFileThread(FILE *pFile)
{
DWORD	uId;

	if(!hAudioId)return -1;

	if(hAudioThread)AudioStop();						// Close old thread

	pAudioCallback  = AudioFileRead;
	hAudioThread	= CreateThread(0,1024,AudioProcThread,pFile,0,&uId);


return 0;
}

//*****************************************************************************
//*
//*		AudioWriteProcThread
//*
//*****************************************************************************
//	This funktion starts an thread which read the Data with a funktion
//	AudioWriteProcThread uses the pProc(pParam,void* Buffer,int Count) to read the Data
//	The Data will be readed in Background
int AudioWriteProcThread(AudioRdProc pProc,void *pParam)
{
DWORD	uId;


	if(!hAudioId)return -1;

	if(hAudioThread)AudioStop();						// Close old thread

	pAudioCallback  = pProc;
	hAudioThread	= CreateThread(0,1024,AudioProcThread,pParam,0,&uId);


return 0;
}
